/* Copyright (C)2006 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2006 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: jash.c,v 1.4 2013/01/26 12:07:53 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <login_cap.h>
#include <sys/time.h>
#include <sys/jail.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <err.h>
#include <pwd.h>
#include <sys/resource.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <mife.h>

#define CONFNAME "/usr/local/etc/jash.rc"
char *outarg[] = {"-", NULL};

int
main(int argc, char **argv) {
    mife_descriptor *config;
    babolo_parm *jparm;
    struct passwd *pent;
    struct in_addr in;
    login_cap_t *lc;
    ssize_t conflen;
    struct jail j;
    u_char *nextrc;
    BLIN_flag flags = 0;
    char *idin;
    int line;
    off_t off;

#ifdef DEBUG
    flags = BLIN_VER1|BLIN_VER2|BLIN_VER3|BLIN_VER4;
#endif
    idin = getlogin();
#ifdef DEBUG
    ifBLIN_QV3(flags) fprintf(stderr, "getlogin %s\n", idin);
#endif
    config = mife_open(MIFE_FULL, CONFNAME);
    if  (!config) {
        ifBLIN_QV1(flags) warn("mife_open");
        exit(EX_IOERR);
    }
    if  (mife_eolinit(config, "\n") < 0) {
        ifBLIN_QV1(flags) warn("mife_eolinit");
        exit(EX_SOFTWARE);
    }
    for (line = 1, off = 0;; line++) {
        if  ((conflen = mife_read(config, 0, off)) < 0) exit(EX_NOINPUT);
        if  (conflen == 0) {
            ifBLIN_QV1(flags) warn("No user");
            exit(EX_NOUSER);
        }
        nextrc = mife_get(config, off);
        if  (!nextrc) {
            ifBLIN_QV1(flags) warn("mife_get");
            exit(EX_IOERR);
        }
        jparm = babolo_getparm(Bpars_FIXB|Bpars_NOEN|Bpars_XXLN|kukBpars_NPRM(5), &nextrc, NULL, 0);
        if  (!jparm) {
            ifBLIN_QV1(flags) warn("babolo_getparm");
            exit(EX_DATAERR);
        }
        if  (jparm->argc < 4) exit(EX_DATAERR);
        if  (!strcmp((char*)jparm->argv[0], idin)) break;
        babolo_freeparm(jparm);
        off = config->offset + (nextrc - (u_char*)config->buffer);
    }
    memset(&j, 0, sizeof(j));
#   ifdef JAIL_API_VERSION
        j.version = JAIL_API_VERSION;
#   else
        j.version = 0;
#   endif
    j.path = (char*)jparm->argv[1];
    j.hostname = (char*)jparm->argv[2];
    if  (!inet_aton((char*)jparm->argv[3], &in)) exit(EX_NOHOST);
#   ifdef JAIL_API_VERSION
        j.ip4s = 1;
        j.ip4 = &in;
#   else
        j.ip_number = ntohl(in.s_addr);
#   endif
    if  (jail(&j)) {
        ifBLIN_QV1(flags) warn("jail");
        exit(EX_SOFTWARE);
    }
    if  (setsid() == -1) {  
        ifBLIN_QV2(flags) warn("setsid");
    }
    if  (jparm->argc >= 4 && setpriority(PRIO_PROCESS, 0, atoi((char*)jparm->argv[4]))) {
        ifBLIN_QV1(flags) warn("setpriority");
        exit(EX_CONFIG);
    }
    if  (!(pent = getpwnam(idin))) {
        ifBLIN_QV1(flags) warn("getpwnam");
        exit(EX_SOFTWARE);
    }
    if  (!(lc = login_getpwclass(pent))) {
        ifBLIN_QV1(flags) warn("login_getpwclass");
        exit(EX_SOFTWARE);
    }
    if  (setusercontext(lc, pent, pent->pw_uid, LOGIN_SETALL) < 0) {
        ifBLIN_QV1(flags) warn("setusercontext");
        exit(EX_SOFTWARE);
    }
    if  (chdir(pent->pw_dir)) {
        ifBLIN_QV2(flags) warn("chdir");
    }
    execv(pent->pw_shell, outarg);
    ifBLIN_QV1(flags) warn("execv");
    exit(EX_SOFTWARE);
}
